----------------------------------------------------------------------------------
--  
--        spi.vhd        MC9S08JSxx  FPGA W[Ԃ̃C^[tF[X
--  
--        (c)2011/03/23  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷Ȃǂ
--  ƓIp͂߂B
--
--
--
--  DataFlash ƎC^[tF[XƂiADR  2 oCgj
--
--    CS_X  L ɂAR}hsf[^o͂܂ōsȂ H ɖ߂B
--    ̓f[^̓NbN̗オœǂݍ܂B
--    o̓f[^̓NbN̗ŕωB
--
--      Read Array      03 ADRS(2) RDATA(1+)
--      Page Program    02 ADRS(2) WDATA(1+)
--
--
--


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;


------------------------------------------------------------------------------
entity spi is
  port (
    RESET   : in  std_logic;
    CLK     : in  std_logic;                      -- I/O bus clock
    
    CS_X    : in  std_logic;                      -- Chip Select 
    SCK     : in  std_logic;                      -- Serial Clock
    SI      : in  std_logic;                      -- Serial IN
    SO      : out std_logic;                      -- Serial OUT
    
    MWE     : out std_logic;                      -- Module Write Enable
    MCE     : out std_logic_vector(15 downto 0 ); -- Module Enable
    MWDATA  : out std_logic_vector( 7 downto 0 ); -- W[݃f[^
    MRDATA  : in  std_logic_vector( 7 downto 0 ); -- W[ǂݏof[^
    MADR    : out std_logic_vector(15 downto 0 ); -- W[AhX
    
    DEBUG   : out std_logic                       -- 
  );
end spi;


------------------------------------------------------------------------------
architecture RTL of spi is
  
  signal SCK_1,SCK_2 : std_logic;                 -- SCK ̓
  signal CS_X_1,CS_X_2 : std_logic;               -- CS ̓
  
  signal CNT_BIT  : std_logic_vector(2 downto 0); -- 
  signal RW       : std_logic;                    -- Read='1'  Write='0'
  
  signal MCE1     : std_logic_vector(15 downto 0);-- 
--  signal MEME1    : std_logic_vector( 7 downto 0);-- 
  signal ADR      : std_logic_vector(15 downto 0);-- 
  signal ADRINC   : std_logic;                    -- AhX +1 ^C~O
  signal ADRCARRY : std_logic;                    -- AhXJE^[̓rL[
  signal RDATA    : std_logic_vector(7 downto 0); -- ǂݏof[^
  signal WDATA    : std_logic_vector(7 downto 0); -- ݃f[^
  signal ND       : std_logic;                    -- f[^ł
  
  signal STATE    : integer range 0 to 3;         -- Xe[g
  
  constant STATE_CMD  : integer := 0;
  constant STATE_ADR1 : integer := 1;
  constant STATE_ADR2 : integer := 2;
  constant STATE_DATA : integer := 3;
  
begin
  
  
  SO      <= RDATA(7);
  DEBUG   <= ADR(0);
  MCE1    <= "0000000000000000" when( CS_X_2='1'              ) else  -- not Selected
             "0000000000000000" when( ADR(15)='1'             ) else  -- memory R/W 0x8000-0xFFFF
             "0000000000000001" when( ADR(11 downto 8) = X"0" ) else  -- Signature 0x00xx
             "0000000000000010" when( ADR(11 downto 8) = X"1" ) else  -- 
             "0000000000000100" when( ADR(11 downto 8) = X"2" ) else  -- 
             "0000000000001000" when( ADR(11 downto 8) = X"3" ) else  -- 
             "0000000000010000" when( ADR(11 downto 8) = X"4" ) else  -- dds 0x04xx
             "0000000000100000" when( ADR(11 downto 8) = X"5" ) else  -- 
             "0000000001000000" when( ADR(11 downto 8) = X"6" ) else  -- 
             "0000000010000000" when( ADR(11 downto 8) = X"7" ) else  -- 
             "0000000100000000" when( ADR(11 downto 8) = X"8" ) else  -- freqcounter 0x08xx
             "0000001000000000" when( ADR(11 downto 8) = X"9" ) else  -- 
             "0000010000000000" when( ADR(11 downto 8) = X"A" ) else  -- 
             "0000100000000000" when( ADR(11 downto 8) = X"B" ) else  -- 
             "0001000000000000" when( ADR(11 downto 8) = X"C" ) else  -- 
             "0010000000000000" when( ADR(11 downto 8) = X"D" ) else  -- 
             "0100000000000000" when( ADR(11 downto 8) = X"E" ) else  -- 
             "1000000000000000" when( ADR(11 downto 8) = X"F" ) else  -- 
             "0000000000000000";                                      -- 
  
  
  process( CLK ) begin
    if( rising_edge(CLK) ) then
      MCE  <= MCE1;
    end if;
  end process;
  
  
  ------------------------------------------------------------------
  --    Xe[g
  ------------------------------------------------------------------
  process( RESET, CLK ) begin
    if ( RESET ='1') then
      CS_X_1 <= '0';
      CS_X_2 <= '0';
    elsif( rising_edge(CLK) ) then
      CS_X_1 <= CS_X;
      CS_X_2 <= CS_X_1;
      
      if( CS_X_1='0' and CS_X_2='1' ) then                     -- CS faling edge
        ADR       <= (others => '0');
        ADRINC    <= '0';
        ADRCARRY  <= '0';
        STATE     <= STATE_CMD;
        RW        <= '0';
      
      elsif( ND = '1' ) then
        case STATE is
          when( STATE_CMD ) =>                  -- command
            if( WDATA(0) = '0' ) then           -- Page Program command( 0x02 )
              RW <= '0';
            else                                -- ׂ͂ Read Array command( 0x03 )Ƃ݂Ȃ
              RW <= '1';
            end if;
            STATE <= STATE_ADR1;
          when( STATE_ADR1 ) =>                 -- adrs15..8
            ADR( 15 downto 8 ) <= WDATA;
            STATE <= STATE_ADR2;
          when( STATE_ADR2 ) =>                 -- adrs7..0
            ADR( 7 downto 0 ) <= WDATA;
            STATE <= STATE_DATA;
          when others =>                        -- read/write data
            ADR(3 downto 0) <= ADR(3 downto 0) + 1;
            if( ADR(3 downto 0) = X"F" ) then
              ADRCARRY <= '1';
            end if;
            if( RW = '0' ) then
              MWDATA <= WDATA;
              MWE    <= '1';
            end if;
        end case;
        
      else
        if( ADRCARRY = '1' ) then
          ADR(15 downto 4) <= ADR(15 downto 4) + 1;
          ADRCARRY <= '0';
        else
          MADR <= ADR;
        end if;
        MWE <= '0';
        
      end if;
      
    end if;
  end process;
  
  
  ------------------------------------------------------------------
  --    f[^o
  ------------------------------------------------------------------
  process( RESET, CLK ) begin
    if ( RESET ='1') then
      SCK_1 <= '0';
      SCK_2 <= '0';
    elsif( rising_edge(CLK) ) then
      SCK_1 <= SCK;
      SCK_2 <= SCK_1;
      
      if( CS_X_1='0' ) then
        
        if( CS_X_2='1' ) then                   -- CS ŃZbg
          WDATA   <= (others => '0');
          CNT_BIT <= (others => '0');
          ND      <= '0';
          
        elsif( SCK_1='1' and SCK_2='0' ) then   -- SCK オ
          WDATA   <= WDATA(6 downto 0) & SI;    -- f[^
          CNT_BIT <= CNT_BIT + 1;
          if( CNT_BIT = "111" ) then            -- 8rbg͂
            ND <= '1';                          -- f[^ł
          end if;
          
        elsif( SCK_1='0' and SCK_2='1' ) then   -- SCK 
          if( CNT_BIT = "000" ) then            -- 
            RDATA <= MRDATA;                    -- ɓ]f[^
          else
            RDATA <= RDATA(6 downto 0) & '0';   -- f[^o
          end if;
          
        else
          ND  <= '0';
        end if;
        
      end if;     -- if( CS_X_1='0' )
      
    end if;
  end process;
  
  
end RTL;
----------------------------- end of architecture ----------------------------
